from typing import List, Tuple




class Embed:
    """Class representing an embed object."""
    def __init__(self, title:str=None, description:str=None, color:int=None, fields:List[Tuple[str, str]]=None, footer:Tuple[str, str]=None):
        self.title:str = title
        self.description:str = description
        self.color:int = color
        self.fields:List = fields
        self.footer:Tuple[str, str] = footer
        self.image:str = None
        self.thumbnail:str = None

    def __repr__(self):
        return f"<Embed title={self.title} description={self.description} color={self.color} fields={self.fields}>"
    
    def __str__(self):
        return self.title
    
    def __eq__(self, other):
        return self.title == other.title
    
    def __ne__(self, other):
        return not self.__eq__(other)
    
    def add_field(self, name:str, value:str):
        self.fields.append((name, value))


    def set_footer(self, text:str, icon:str):
        self.footer = (text, icon)


    def set_image(self, url:str):
        self.image = url

    def to_dict(self):
        return {
            "title":self.title,
            "description":self.description,
            "color":self.color,
            "fields":self.fields,
            "footer":self.footer,
            "image":self.image,
            "thumbnail":self.thumbnail
        }
    




# from ..models.base import DiscordModelsBase
# from ..models import Guild


class Role():
    ROUTE = "/guilds/{guild_id}/roles"
    def __init__(self, _payload:dict, guild) -> None:
        self.guild = guild
        self.name = _payload.get("name")
        self.id = int(_payload.get("id"))
        self.color = _payload.get("color")
        self.hoist = _payload.get("hoist")
        self.tags = _payload.get("tags")
        self.icon_hash = _payload.get("icon")
        self.members = _payload.get("members")
        self.managed = _payload.get("managed")
        self.position = _payload.get("position")
        self.description = _payload.get("description")
        self.permissions = _payload.get("permissions")
        self.mentionable = _payload.get("mentionable")
        self.accent_color = _payload.get("accent_color")
        self.integration_id = _payload.get("integration_id")
        self.permissions_new = _payload.get("permissions_new")




import os

from flask import Flask, redirect, url_for
from flaskcord import Session, requires_authorization


app = Flask(__name__)

app.secret_key = b"%\xe0'\x01\xdeH\x8e\x85m|\xb3\xffCN\xc9g"
os.environ["OAUTHLIB_INSECURE_TRANSPORT"] = "true"    # !! Only in development environment.

app.config["DISCORD_CLIENT_ID"] = 490732332240863233
app.config["DISCORD_CLIENT_SECRET"] = os.getenv("DISCORD_CLIENT_SECRET")
app.config["DISCORD_BOT_TOKEN"] = os.getenv("DISCORD_BOT_TOKEN")
app.config["DISCORD_REDIRECT_URI"] = "http://127.0.0.1:5000/callback"

discord = Session(app)


HYPERLINK = '<a href="{}">{}</a>'


def welcome_user(user):
    dm_channel = discord.bot_request("/users/@me/channels", "POST", json={"recipient_id": user.id})
    return discord.bot_request(
        f"/channels/{dm_channel['id']}/messages", "POST", json={"content": "Thanks for authorizing the app!"}
    )


@app.route("/")
def index():
    if not discord.authorized:
        return f"""
        {HYPERLINK.format(url_for(".login"), "Login")} <br />
        {HYPERLINK.format(url_for(".login_with_data"), "Login with custom data")} <br />
        {HYPERLINK.format(url_for(".invite_bot"), "Invite Bot with permissions 8")} <br />
        {HYPERLINK.format(url_for(".invite_oauth"), "Authorize with oauth and bot invite")}
        """

    return f"""
    {HYPERLINK.format(url_for(".me"), "@ME")}<br />
    {HYPERLINK.format(url_for(".logout"), "Logout")}<br />
    {HYPERLINK.format(url_for(".user_guilds"), "My Servers")}<br />
    {HYPERLINK.format(url_for(".add_to_guild", guild_id=475549041741135881), "Add me to 475549041741135881.")}    
    """


@app.route("/login/")
def login():
    return discord.create_session()


@app.route("/login-data/")
def login_with_data():
    return discord.create_session(data=dict(redirect="/me/", coupon="15off", number=15, zero=0, status=False))


@app.route("/invite-bot/")
def invite_bot():
    return discord.create_session(scope=["bot"], permissions=8, guild_id=464488012328468480, disable_guild_select=True)


@app.route("/invite-oauth/")
def invite_oauth():
    return discord.create_session(scope=["bot", "identify"], permissions=8)


@app.route("/callback/")
def callback():
    data = discord.callback()
    redirect_to = data.get("redirect", "/")

    user = discord.fetch_user()
    welcome_user(user)

    return redirect(redirect_to)


@app.route("/me/")
def me():
    user = discord.fetch_user()
    return f"""
<html>
<head>
<title>{user.name}</title>
</head>
<body><img src='{user.avatar_url or user.default_avatar_url}' />
<p>Is avatar animated: {str(user.is_avatar_animated)}</p>
<a href={url_for("my_connections")}>Connections</a>
<br />
</body>
</html>

"""


@app.route("/me/guilds/")
def user_guilds():
    guilds = discord.fetch_guilds()
    return "<br />".join([f"[ADMIN] {g.name}" if g.permissions.administrator else g.name for g in guilds])


@app.route("/add_to/<int:guild_id>/")
def add_to_guild(guild_id):
    user = discord.fetch_user()
    return user.add_to_guild(guild_id)


@app.route("/me/connections/")
def my_connections():
    user = discord.fetch_user()
    connections = discord.fetch_connections()
    return f"""
<html>
<head>
<title>{user.name}</title>
</head>
<body>
{str([f"{connection.name} - {connection.type}" for connection in connections])}
</body>
</html>

"""


@app.route("/logout/")
def logout():
    discord.revoke()
    return redirect(url_for(".index"))


@app.route("/secret/")
@requires_authorization
def secret():
    return os.urandom(16)


# if __name__ == "__main__":
#     app.run(debug=True)



from .. import configs

class Emoji:
    def init(self, payload:dict):
        self.id = payload.get("id")
        self.name = payload.get("name")
        self.roles = payload.get("roles")
        self.user = payload.get("user")
        self.require_colons = payload.get("require_colons")
        self.managed = payload.get("managed")
        self.animated = payload.get("animated")
        self.available = payload.get("available")

    def __str__(self):
        return self.name
    

    def url(self):
        return f"{configs.DISCORD_EMOJI_URL}{self.id}"


class Integration(object):

    def __init__(self, payload:dict):
        self._payload = payload
        self.id = int(self._payload.get("id", 0))
        self.name = self._payload.get("name")
        self.type = self._payload.get("type")
        self.enabled = self._payload.get("enabled")
        self.syncing = self._payload.get("syncing")
        self.role_id = int(self._payload.get("role_id", 0))
        self.expire_behaviour = self._payload.get("expire_behaviour")
        self.expire_grace_period = self._payload.get("expire_grace_period")
        # self.user = User(self._payload.get("user", dict()))
        self.account = self._payload.get("account")
        self.synced_at = self._payload.get("synced_at")



import os

from flask import Flask
from flaskcord import Session
discord = Session(client_id=490732332240863233)

def get_app():
    app = Flask(__name__)
    app.secret_key = b"%\xe0'\x01\xdeH\x8e\x85m|\xb3\xffCN\xc9g"
    os.environ["OAUTHLIB_INSECURE_TRANSPORT"] = "true"  # !! Only in development environment.
    # app.config["DISCORD_CLIENT_ID"] = 490732332240863233
    app.config["DISCORD_CLIENT_SECRET"] = os.getenv("DISCORD_CLIENT_SECRET")
    app.config["DISCORD_BOT_TOKEN"] = os.getenv("DISCORD_BOT_TOKEN")
    app.config["DISCORD_REDIRECT_URI"] = "http://127.0.0.1:5000/callback"
    discord.init_app(app)

    return app



"""MIT License

Copyright (c) 2019 thecosmos
Copyright (c) 2024 hunter87ff

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE."""





from flask import current_app
from abc import ABCMeta, abstractmethod
import typing

class DiscordModelsMeta(ABCMeta):

    ROUTE = str()

    def __init__(cls, name, *args, **kwargs):
        if not cls.ROUTE and name != "DiscordModelsBase":
            raise NotImplementedError(f"ROUTE must be specified in a Discord model: {name}.")
        super().__init__(name, *args, **kwargs)


class DiscordModelsBase(metaclass=DiscordModelsMeta):

    BOT = False
    MANY = False

    @abstractmethod
    def __init__(self, payload):
        self._payload:dict = payload

    @staticmethod
    def _request(*args, **kwargs):
        """A shorthand to :py:func:flaskcord.request`. It uses Flask current_app local proxy to get the
        flaskcord client.

        """
        return current_app.discord.request(*args, **kwargs)

    @staticmethod
    def _bot_request(route: str, method="GET", **kwargs) -> typing.Union[dict, str]:
        """A shorthand to :py:func:flaskcord.bot_request`."""
        return current_app.discord.bot_request(route, method, **kwargs)

    @classmethod
    def fetch_from_api(cls):
        """A class method which returns an instance or list of instances of this model by implicitly making an
        API call to Discord.

        Returns
        -------
        cls
            An instance of this model itself.
        [cls, ...]
            List of instances of this model when many of these models exist.

        """
        request_method = cls._bot_request if cls.BOT else cls._request
        payload = request_method(cls.ROUTE)
        if cls.MANY:
            return [cls(_) for _ in payload]
        return cls(payload)

    def to_json(self):
        """A utility method which returns raw payload object as it was received from discord.

        Returns
        -------
        dict
            A dict representing raw payload object received from discord.

        """
        return self._payload
